use feature 'say';

say '1..4';

say 'ok 1 - just a normal like without comments';
# say 'not ok - should not print this';
say 'ok 2 - it seems that comments works on end of lines'; # say 'not ok but not!';

#
# say 'no ok , comment block';
#

say 'ok 3 - after block';

# say 'not ok ';
say 'ok 4 - middle of block';
# say 'not ok ';
