use feature 'say';

say  "1..6\n";

say  "ok - Integer: ", 1;
say  "ok - not small integer: ", 67000;
say  "ok - zero: ", 0;

say  "ok - racional: ", 1.29;
say  "ok - negative integer: ", -1;
say  "ok - negative racional: ", -3.14;
